@@ -31,6 +31,7 @@ use nu_errors::ShellError;
 use nu_parser::ParserScope;
 use nu_protocol::{hir::ExternalRedirection, UntaggedValue, Value};
 
+use log::trace;
 use std::error::Error;
 use std::iter::Iterator;
 use std::path::PathBuf;
@@ -78,7 +79,7 @@ pub async fn run_script_file(
         syncer.sync_path_vars(ctx);
 
         if let Err(reason) = syncer.autoenv(ctx) {
-            print_err(reason, &Text::from(""));
+            print_err(reason, &Text::from(""), ctx);
         }
 
         let _ = register_plugins(ctx);
@@ -106,7 +107,7 @@ pub async fn cli(mut context: EvaluationContext) -> Result<(), Box<dyn Error>> {
         syncer.sync_path_vars(ctx);
 
         if let Err(reason) = syncer.autoenv(ctx) {
-            print_err(reason, &Text::from(""));
+            print_err(reason, &Text::from(""), ctx);
         }
 
         let _ = configure_ctrl_c(ctx);
@@ -116,7 +117,19 @@ pub async fn cli(mut context: EvaluationContext) -> Result<(), Box<dyn Error>> {
         rl.set_helper(helper);
     });
 
+    // start time for command duration
+    let startup_commands_start_time = std::time::Instant::now();
+    // run the startup commands
     let _ = run_startup_commands(&mut context, &configuration).await;
+    // Store cmd duration in an env var
+    context.scope.add_env_var(
+        "CMD_DURATION",
+        format!("{:?}", startup_commands_start_time.elapsed()),
+    );
+    trace!(
+        "startup commands took {:?}",
+        startup_commands_start_time.elapsed()
+    );
 
     // Give ourselves a scope to work in
     context.scope.enter_scope();
@@ -140,7 +153,7 @@ pub async fn cli(mut context: EvaluationContext) -> Result<(), Box<dyn Error>> {
 
     #[cfg(windows)]
     {
-        let _ = ansi_term::enable_ansi_support();
+        let _ = nu_ansi_term::enable_ansi_support();
     }
 
     let mut ctrlcbreak = false;
@@ -186,14 +199,14 @@ pub async fn cli(mut context: EvaluationContext) -> Result<(), Box<dyn Error>> {
                                 }
                             }
                             Err(e) => {
-                                crate::cli::print_err(e, &Text::from(prompt_line));
+                                crate::cli::print_err(e, &Text::from(prompt_line), &context);
                                 context.clear_errors();
 
                                 "> ".to_string()
                             }
                         },
                         Err(e) => {
-                            crate::cli::print_err(e, &Text::from(prompt_line));
+                            crate::cli::print_err(e, &Text::from(prompt_line), &context);
                             context.clear_errors();
 
                             "> ".to_string()
@@ -227,6 +240,9 @@ pub async fn cli(mut context: EvaluationContext) -> Result<(), Box<dyn Error>> {
             session_text.push('\n');
         }
 
+        // start time for command duration
+        let cmd_start_time = std::time::Instant::now();
+
         let line = match convert_rustyline_result_to_string(readline) {
             LineResult::Success(_) => {
                 process_script(
@@ -241,6 +257,11 @@ pub async fn cli(mut context: EvaluationContext) -> Result<(), Box<dyn Error>> {
             x => x,
         };
 
+        // Store cmd duration in an env var
+        context
+            .scope
+            .add_env_var("CMD_DURATION", format!("{:?}", cmd_start_time.elapsed()));
+
         // Check the config to see if we need to update the path
         // TODO: make sure config is cached so we don't path this load every call
         // FIXME: we probably want to be a bit more graceful if we can't set the environment
@@ -253,7 +274,7 @@ pub async fn cli(mut context: EvaluationContext) -> Result<(), Box<dyn Error>> {
             }
 
             if let Err(reason) = syncer.autoenv(ctx) {
-                print_err(reason, &Text::from(""));
+                print_err(reason, &Text::from(""), ctx);
             }
 
             let _ = configure_rustyline_editor(&mut rl, config);
@@ -275,9 +296,7 @@ pub async fn cli(mut context: EvaluationContext) -> Result<(), Box<dyn Error>> {
                 rl.add_history_entry(&line);
                 let _ = rl.save_history(&history_path);
 
-                context.with_host(|_host| {
-                    print_err(err, &Text::from(session_text.clone()));
-                });
+                print_err(err, &Text::from(session_text.clone()), &context);
 
                 maybe_print_errors(&context, Text::from(session_text.clone()));
             }
