@@ -14,45 +14,23 @@ extern crate quickcheck;
 extern crate quickcheck_macros;
 
 mod cli;
-mod commands;
 #[cfg(feature = "rustyline-support")]
 mod completion;
-mod deserializer;
-mod documentation;
 mod env;
-mod evaluate;
-mod evaluation_context;
 mod format;
-mod futures;
-#[cfg(feature = "rustyline-support")]
-mod git;
 #[cfg(feature = "rustyline-support")]
 mod keybinding;
-mod path;
-mod plugin;
-pub mod script;
+mod line_editor;
 mod shell;
 pub mod types;
-pub mod utils;
-
-#[cfg(test)]
-mod examples;
 
 #[cfg(feature = "rustyline-support")]
 pub use crate::cli::cli;
 
 pub use crate::cli::{parse_and_eval, register_plugins, run_script_file};
-pub use crate::commands::classified::block::run_block;
-pub use crate::commands::command::{
-    whole_stream_command, CommandArgs, EvaluatedWholeStreamCommandArgs, Example, WholeStreamCommand,
-};
 
-pub use crate::commands::default_context::create_default_context;
-pub use crate::commands::help::get_help;
 pub use crate::env::environment_syncer::EnvironmentSyncer;
-pub use crate::env::host::BasicHost;
-pub use crate::evaluation_context::EvaluationContext;
-pub use crate::prelude::ToOutputStream;
+pub use nu_command::commands::default_context::create_default_context;
 pub use nu_data::config;
 pub use nu_data::dict::TaggedListBuilder;
 pub use nu_data::primitive;
